/* ElGamal using GMP

   main.c

   Copyright (C) 1999/2000 Christian Koch
   ckoch@et.htwk-leipzig.de

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; see the file COPYING.  If not, write to the Free Software
Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */



#include <windows.h>
#include <windowsx.h>
#include <commdlg.h>
#include <stdlib.h>
#include <gmp.h>
#include "global.h"
#include "main.h"
#include "fktn.h"
#include "id.h"


HINSTANCE hActInstance;
char szAppName[] = "ElGamal";

/* Zahlenbasis fr Aus- und Eingabe der Parameter */
int BASE;

/* Schlsselpaar */
PubKey pk;
SecKey sk;



int WINAPI
WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR szCmdLine,\
        int iCmdShow)
{
    hActInstance = hInstance;

    /* auf Befehlszeilenargument berprfen */
    BASE = atoi(szCmdLine);
    if ((BASE < 2) || (BASE > 36))
    {
        BASE = 10;
    }

    return DialogBox(hInstance, szAppName, NULL, DlgProc);
}



BOOL CALLBACK
DlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        HANDLE_MSG(hwnd, WM_INITDIALOG, Cls_OnInitDialog);
        HANDLE_MSG(hwnd, WM_COMMAND, Cls_OnCommand);
        HANDLE_MSG(hwnd, WM_SYSCOMMAND, Cls_OnSysCommand);
    }

    return FALSE;
}



BOOL
Cls_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
    HICON hIcon = LoadIcon(hActInstance, szAppName);

    SendMessage(hwnd, WM_SETICON, (WPARAM) ICON_SMALL, (LPARAM) hIcon);

    /* Initialisierung des Schlsselpaars */
    mpz_init(pk.p);
    mpz_init(pk.alpha);
    mpz_init(pk.beta);
    mpz_init(sk.p);
    mpz_init(sk.a);

    return FALSE;
}



void
Cls_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
    char szbuf[MAX_BIT + 1], *buf_p;
    int i, j, L, str_len, no_of_blocks;
    mpz_t *mpztemp_p = NULL, *mpztemp2_p;
    OPENFILENAME sfn;
    HANDLE hFile;
    DWORD dwBytes;


    switch (id)
    {
        case IDB_neu:
        /* Bitlnge bestimmen und auf Grenzen berprfen */
        L = GetDlgItemInt(hwnd, IDE_Bitlaenge, NULL, FALSE);
        if ((L < 20) || (L > MAX_BIT))
        {
             MessageBox(hwnd, "20 <= Bitlnge <= 200", szAppName,\
                        MB_OK | MB_ICONINFORMATION);
             break;
        }
        /* Schlsselpaar der Lnge L Bit erzeugen */
        genkeypair(&pk, &sk, L);

        /* Schlsseldaten in die Editboxen eintragen */
        mpz_get_str(szbuf, BASE, pk.p);
        SetDlgItemText(hwnd, IDE_p, szbuf);

        mpz_get_str(szbuf, BASE, pk.alpha);
        SetDlgItemText(hwnd, IDE_alpha, szbuf);

        mpz_get_str(szbuf, BASE, sk.a);
        SetDlgItemText(hwnd, IDE_a, szbuf);

        mpz_get_str(szbuf, BASE, pk.beta);
        SetDlgItemText(hwnd, IDE_beta, szbuf);
        break;


        case IDB_Chiffrieren:
        /* alpha aus Editbox auslesen */
        str_len = SendDlgItemMessage(hwnd, IDE_alpha, WM_GETTEXTLENGTH, 0, 0);
        if (str_len == 0)
        {
            MessageBox(hwnd, "Bitte alpha eingeben.", szAppName,\
                       MB_OK | MB_ICONINFORMATION);
            break;
        }
        GetDlgItemText(hwnd, IDE_alpha, szbuf, sizeof (szbuf));
        mpz_set_str(pk.alpha, szbuf, BASE);

        /* beta aus Editbox auslesen */
        str_len = SendDlgItemMessage(hwnd, IDE_beta, WM_GETTEXTLENGTH, 0, 0);
        if (str_len == 0)
        {
            MessageBox(hwnd, "Bitte beta eingeben.", szAppName,\
                       MB_OK | MB_ICONINFORMATION);
            break;
        }
        GetDlgItemText(hwnd, IDE_beta, szbuf, sizeof (szbuf));
        mpz_set_str(pk.beta, szbuf, BASE);

        /* p aus Editbox auslesen */
        str_len = SendDlgItemMessage(hwnd, IDE_p, WM_GETTEXTLENGTH, 0, 0);
        if (str_len == 0)
        {
            MessageBox(hwnd, "Bitte p eingeben.", szAppName,\
                       MB_OK | MB_ICONINFORMATION);
            break;
        }
        GetDlgItemText(hwnd, IDE_p, szbuf, sizeof (szbuf));
        mpz_set_str(pk.p, szbuf, BASE);

        /* L aus p bestimmen und in Editbox eintragen */
        L = mpz_sizeinbase(pk.p, 2);
        SetDlgItemInt(hwnd, IDE_Bitlaenge, L, FALSE);

        /* Bestimmung der Lnge des Klartextstrings */
        str_len = SendDlgItemMessage(hwnd, IDE_Klartext, WM_GETTEXTLENGTH, 0, 0);
        if (str_len == 0)
        {
            MessageBox(hwnd, "Bitte Klartext eingeben.", szAppName,\
                       MB_OK | MB_ICONINFORMATION);
            break;
        }

        /* Anfordern von Speicher fr den Klartextstring */
        buf_p = (char *) VirtualAlloc(NULL, ++str_len * sizeof (char),\
                                      MEM_COMMIT, PAGE_READWRITE);

        /* Klartext aus Editbox auslesen */
        GetDlgItemText(hwnd, IDE_Klartext, buf_p, str_len);

        /* Klartext in Blcke der Lnge L-1 aufteilen */
        mpztemp_p = str2arr(&no_of_blocks, buf_p, L-1);
        VirtualFree(buf_p, 0, MEM_RELEASE);

        /* Klartextblcke mit pk verschlsseln */
        mpztemp2_p = encrypt(mpztemp_p, pk, no_of_blocks);

        /* Freigeben des Speichers der Klartextblcke */
        for (i = 0; i < no_of_blocks; i++)
        {
            mpz_clear(mpztemp_p[i]);
        }
        VirtualFree(mpztemp_p, 0, MEM_RELEASE);

        /* Bestimmung der Lnge des Geheimtextstrings */
        no_of_blocks *= 2;
        str_len = 0;
        for (i = 0; i < no_of_blocks; i++)
        {
            str_len += mpz_sizeinbase(mpztemp2_p[i], BASE);
        }

        /* Anfordern von Speicher fr den Geheimtextstring */
        buf_p = (char *) VirtualAlloc(NULL, (++str_len + no_of_blocks) *\
                                             sizeof (char),\
                                             MEM_COMMIT, PAGE_READWRITE);
        /* buf_p[0] = '\0'; VirtualAlloc initialisiert Speicher mit 0 */

        /* Umwandlung der Geheimtextblcke in den Geheimtextstring */
        for (i = 0; i < no_of_blocks; i++)
        {
            mpz_get_str(szbuf, BASE, mpztemp2_p[i]);
            lstrcat(buf_p, szbuf);
            lstrcat(buf_p, "_");
        }

        /* Freigeben des Speichers der Geheimtextblcke */
        for (i = 0; i < no_of_blocks; i++)
        {
            mpz_clear(mpztemp2_p[i]);
        }
        VirtualFree(mpztemp2_p, 0, MEM_RELEASE);

        /* Ausgabe der Geheimtextstrings */
        SetDlgItemText(hwnd, IDE_Geheimtext, buf_p);

        /* Freigabe des Speichers des Geheimtextstrings */
        VirtualFree(buf_p, 0, MEM_RELEASE);
        break;


        case IDB_Dechiffrieren:
        /* p aus Editbox auslesen */
        str_len = SendDlgItemMessage(hwnd, IDE_p, WM_GETTEXTLENGTH, 0, 0);
        if (str_len == 0)
        {
            MessageBox(hwnd, "Bitte p eingeben.", szAppName,\
                       MB_OK | MB_ICONINFORMATION);
            break;
        }
        GetDlgItemText(hwnd, IDE_p, szbuf, ++str_len);
        mpz_set_str(sk.p, szbuf, BASE);

        /* a aus Editbox auslesen */
        str_len = SendDlgItemMessage(hwnd, IDE_a, WM_GETTEXTLENGTH, 0, 0);
        if (str_len == 0)
        {
            MessageBox(hwnd, "Bitte a eingeben.", szAppName,\
                       MB_OK | MB_ICONINFORMATION);
            break;
        }
        GetDlgItemText(hwnd, IDE_a, szbuf, ++str_len);
        mpz_set_str(sk.a, szbuf, BASE);

        /* L aus p bestimmen und in Editbox eintragen */
        L = mpz_sizeinbase(sk.p, 2);
        SetDlgItemInt(hwnd, IDE_Bitlaenge, L, FALSE);
        if ((L < 20) || (L > MAX_BIT))
        {
             MessageBox(hwnd, "20 <= Bitlnge <= 200", szAppName,\
                        MB_OK | MB_ICONINFORMATION);
             break;
        }

        /* Bestimmung der Lnge des Geheimtextes */
        str_len = SendDlgItemMessage(hwnd, IDE_Geheimtext, WM_GETTEXTLENGTH, 0, 0);
        if (str_len == 0)
        {
            MessageBox(hwnd, "Bitte Geheimtext eingeben.", szAppName,\
                       MB_OK | MB_ICONINFORMATION);
            break;
        }

        /* Anfordern von Speicher fr den Geheimtextstring */
        buf_p = (char *) VirtualAlloc(NULL, ++str_len * sizeof (char),\
                                      MEM_COMMIT, PAGE_READWRITE);

        /* Geheimtext aus Editbox auslesen */
        GetDlgItemText(hwnd, IDE_Geheimtext, buf_p, str_len);

        /* Bestimmung der Anzahl der Geheimtextblcke */
        no_of_blocks = 0;
        i = 0;
        while (buf_p[i] != '\0')
        {
            if (buf_p[i] == '_')
            {
                no_of_blocks++;
            }
            i++;
        }

        /* Anfordern von Speicher fr die Geheimtextblcke */
        mpztemp2_p = (mpz_t *) VirtualAlloc(NULL, no_of_blocks * sizeof (mpz_t),\
                                            MEM_COMMIT, PAGE_READWRITE);

        /* Umwandlung des Geheimtextstrings in die Geheimtextblcke */
        j = 0;
        for (i = 0; i < no_of_blocks; i++)
        {
            str_len = 0;
            while (buf_p[j+str_len] != '_')
            {
                str_len++;
            }
            buf_p[j+str_len] = '\0';
            mpz_init(mpztemp2_p[i]);
            mpz_set_str(mpztemp2_p[i], &buf_p[j], BASE);
            j += str_len + 1;
        }

        /* Freigeben des Speichers des Geheimtextstrings */
        VirtualFree(buf_p, 0, MEM_RELEASE);

        /* Geheimtextblcke mit sk entschlsseln */
        no_of_blocks /= 2;
        mpztemp_p = decrypt(mpztemp2_p, sk, no_of_blocks);

        /* Freigeben des Speichers der Geheimtextblcke */
        for (i = 0; i < 2 * no_of_blocks; i++)
        {
            mpz_clear(mpztemp2_p[i]);
        }
        VirtualFree(mpztemp2_p, 0, MEM_RELEASE);

        /* Klartextblcke zum Klartextstring zusammensetzen */
        buf_p = arr2str(no_of_blocks, mpztemp_p, L-1);

        /* Freigeben des Speichers der Klartextblcke */
        for (i = 0; i < no_of_blocks; i++)
        {
            mpz_clear(mpztemp_p[i]);
        }
        VirtualFree(mpztemp_p, 0, MEM_RELEASE);

        /* Ausgabe des Klartextstrings */
        SetDlgItemText(hwnd, IDE_Klartext, buf_p);

        /* Freigabe des Speichers des Klartextstrings */
        VirtualFree(buf_p, 0, MEM_RELEASE);
        break;


        case IDB_SPK:
        ZeroMemory(&sfn, sizeof (OPENFILENAME));

        lstrcpy(szbuf, "public_key");
        sfn.lStructSize = sizeof (OPENFILENAME);
        sfn.hwndOwner = hwnd;
        sfn.lpstrFilter = "Public-Key (*.txt)\0*.txt\0";
        sfn.lpstrFile = szbuf;
        sfn.nMaxFile = sizeof (szbuf);
        sfn.Flags = OFN_EXPLORER | OFN_HIDEREADONLY |\
                    OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST;
        sfn.lpstrDefExt = "pk";

        if (!GetSaveFileName(&sfn))
        {
            break;
        }

        hFile = CreateFile(szbuf, GENERIC_WRITE, FILE_SHARE_WRITE, NULL,
                           CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
        if (hFile == INVALID_HANDLE_VALUE)
        {
            break;
        }

        /* Anfordern von Speicher fr den Ausgabestring */
        buf_p = (char *) VirtualAlloc(NULL, 4 * MAX_BIT * sizeof (char),\
                                      MEM_COMMIT, PAGE_READWRITE);

        /* Schlsselparameter auslesen */
        lstrcpy(buf_p, "p     = ");
        GetDlgItemText(hwnd, IDE_p, szbuf, sizeof (szbuf));
        lstrcat(buf_p, szbuf);

        lstrcat(buf_p, "\r\nalpha = ");
        GetDlgItemText(hwnd, IDE_alpha, szbuf, sizeof (szbuf));
        lstrcat(buf_p, szbuf);

        lstrcat(buf_p, "\r\nbeta  = ");
        GetDlgItemText(hwnd, IDE_beta, szbuf, sizeof (szbuf));
        lstrcat(buf_p, szbuf);

        /* Schlsselparameter in Datei schreiben */
        WriteFile (hFile, buf_p, lstrlen(buf_p), &dwBytes, NULL);

        /* Datei schlieen und Speicher freigeben */
        CloseHandle (hFile);
        VirtualFree(buf_p, 0, MEM_RELEASE);

        break;


        case IDB_SGT:
        ZeroMemory(&sfn, sizeof (OPENFILENAME));

        lstrcpy(szbuf, "geheim");
        sfn.lStructSize = sizeof (OPENFILENAME);
        sfn.hwndOwner = hwnd;
        sfn.lpstrFilter = "Geheimtext (*.txt)\0*.txt\0";
        sfn.lpstrFile = szbuf;
        sfn.nMaxFile = sizeof (szbuf);
        sfn.Flags = OFN_EXPLORER | OFN_HIDEREADONLY |\
                    OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST;
        sfn.lpstrDefExt = "txt";

        if (!GetSaveFileName(&sfn))
        {
            break;
        }

        hFile = CreateFile(szbuf, GENERIC_WRITE, FILE_SHARE_WRITE, NULL,
                           CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
        if (hFile == INVALID_HANDLE_VALUE)
        {
            break;
        }

        /* Bestimmung der Lnge des Geheimtextes */
        str_len = SendDlgItemMessage(hwnd, IDE_Geheimtext, WM_GETTEXTLENGTH, 0, 0);

        /* Anfordern von Speicher fr den Geheimtextstring */
        buf_p = (char *) VirtualAlloc(NULL, ++str_len * sizeof (char),\
                                      MEM_COMMIT, PAGE_READWRITE);

        /* Geheimtext aus Editbox auslesen */
        GetDlgItemText(hwnd, IDE_Geheimtext, buf_p, str_len);

        /* Geheimtext in Datei schreiben */
        WriteFile (hFile, buf_p, str_len, &dwBytes, NULL);

        /* Datei schlieen und Speicher freigeben */
        CloseHandle (hFile);
        VirtualFree(buf_p, 0, MEM_RELEASE);

        break;


        case IDCANCEL:
        mpz_clear(pk.p);
        mpz_clear(pk.alpha);
        mpz_clear(pk.beta);
        mpz_clear(sk.p);
        mpz_clear(sk.a);

        EndDialog(hwnd, 0);
        break;
    }
}



void
Cls_OnSysCommand(HWND hwnd, UINT cmd, int x, int y)
{
    switch (LOWORD (cmd))
    {
        case SC_CLOSE:
        mpz_clear(pk.p);
        mpz_clear(pk.alpha);
        mpz_clear(pk.beta);
        mpz_clear(sk.p);
        mpz_clear(sk.a);

        EndDialog(hwnd, 0);
    }
}
